@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -39,12 +39,12 @@
 #define AUDIO_DEVICE_STATE_LEN	19	/* strlen(AUDIO_DEVICE_STATE) */
 #define AUDIO_BUFFER_SIZE	2*MAXPATHLEN
 
-static AudioFormat audio_format = { 0, 0, 0 };
+static AudioFormat audio_format;
 
-static AudioFormat *audio_configFormat = NULL;
+static AudioFormat *audio_configFormat;
 
-static AudioOutput *audioOutputArray = NULL;
-static mpd_uint8 audioOutputArraySize = 0;
+static AudioOutput *audioOutputArray;
+static mpd_uint8 audioOutputArraySize;
 
 #define	DEVICE_OFF        0x00
 #define DEVICE_ENABLE	  0x01   /* currently off, but to be turned on */
@@ -53,13 +53,13 @@ static mpd_uint8 audioOutputArraySize = 0;
 
 /* the audioEnabledArray should be stuck into shared memory, and then disable
    and enable in playAudio() routine */
-static mpd_uint8 *audioDeviceStates = NULL;
+static mpd_uint8 *audioDeviceStates;
 
-static mpd_uint8 audioOpened = 0;
+static mpd_uint8 audioOpened;
 
-static mpd_sint32 audioBufferSize = 0;
-static char *audioBuffer = NULL;
-static mpd_sint32 audioBufferPos = 0;
+static mpd_sint32 audioBufferSize;
+static char *audioBuffer;
+static mpd_sint32 audioBufferPos;
 
 size_t audio_device_count(void)
 {
@@ -89,14 +89,6 @@ int cmpAudioFormat(AudioFormat * f1, AudioFormat * f2)
 	return 1;
 }
 
-extern AudioOutputPlugin alsaPlugin;
-extern AudioOutputPlugin aoPlugin;
-extern AudioOutputPlugin ossPlugin;
-extern AudioOutputPlugin osxPlugin;
-extern AudioOutputPlugin pulsePlugin;
-extern AudioOutputPlugin mvpPlugin;
-extern AudioOutputPlugin shoutPlugin;
-
 void loadAudioDrivers(void)
 {
 	initAudioOutputPlugins();
@@ -107,6 +99,7 @@ void loadAudioDrivers(void)
 	loadAudioOutputPlugin(&pulsePlugin);
 	loadAudioOutputPlugin(&mvpPlugin);
 	loadAudioOutputPlugin(&shoutPlugin);
+	loadAudioOutputPlugin(&jackPlugin);
 }
 
 /* make sure initPlayerData is called before this function!! */
@@ -134,23 +127,21 @@ void initAudioDriver(void)
 		if (!initAudioOutput(output, param)) {
 			if (param)
 			{
-				ERROR("problems configuring output device "
+				FATAL("problems configuring output device "
 				      "defined at line %i\n", param->line);
 			}
 			else
 			{
-				ERROR("No audio_output specified and unable to "
+				FATAL("No audio_output specified and unable to "
 				      "detect a default audio output device\n");
 			}
-			exit(EXIT_FAILURE);
 		}
 
 		/* require output names to be unique: */
 		for (j = 0; j < i; j++) {
 			if (!strcmp(output->name, audioOutputArray[j].name)) {
-				ERROR("output devices with identical "
+				FATAL("output devices with identical "
 				      "names: %s\n", output->name);
-				exit(EXIT_FAILURE);
 			}
 		}
 		audioDeviceStates[i] = DEVICE_ENABLE;
@@ -176,9 +167,8 @@ void initAudioConfig(void)
 	audio_configFormat = xmalloc(sizeof(AudioFormat));
 
 	if (0 != parseAudioConfig(audio_configFormat, param->value)) {
-		ERROR("error parsing \"%s\" at line %i\n",
+		FATAL("error parsing \"%s\" at line %i\n",
 		      CONF_AUDIO_OUTPUT_FORMAT, param->line);
-		exit(EXIT_FAILURE);
 	}
 }
 
