@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,9 +28,9 @@
 #include "permission.h"
 #include "buffer2array.h"
 #include "log.h"
-#include "dbUtils.h"
 #include "tag.h"
 #include "utils.h"
+#include "storedPlaylist.h"
 
 #include <assert.h>
 #include <stdarg.h>
@@ -92,6 +92,15 @@
 #define COMMAND_DEVICES		"outputs"
 #define COMMAND_COMMANDS	"commands"
 #define COMMAND_NOTCOMMANDS	"notcommands"
+#define COMMAND_PLAYLISTCLEAR   "playlistclear"
+#define COMMAND_PLAYLISTADD	"playlistadd"
+#define COMMAND_PLAYLISTFIND	"playlistfind"
+#define COMMAND_PLAYLISTSEARCH	"playlistsearch"
+#define COMMAND_PLAYLISTMOVE	"playlistmove"
+#define COMMAND_PLAYLISTDELETE	"playlistdelete"
+#define COMMAND_TAGTYPES	"tagtypes"
+#define COMMAND_COUNT		"count"
+#define COMMAND_RENAME		"rename"
 
 #define COMMAND_STATUS_VOLUME           "volume"
 #define COMMAND_STATUS_STATE            "state"
@@ -132,8 +141,8 @@ struct _CommandEntry {
 	CommandListHandlerFunction listHandler;
 };
 
-static char *current_command = NULL;
-static int command_listNum = 0;
+static char *current_command;
+static int command_listNum;
 
 static CommandEntry *getCommandEntryFromString(char *string, int *permission);
 
@@ -174,6 +183,12 @@ static int handleUrlHandlers(int fd, int *permission, int argc, char *argv[])
 	return printRemoteUrlHandlers(fd);
 }
 
+static int handleTagTypes(int fd, int *permission, int argc, char *argv[])
+{
+	printTagTypes(fd);
+	return 0;
+}
+
 static int handlePlay(int fd, int *permission, int argc, char *argv[])
 {
 	int song = -1;
@@ -388,17 +403,18 @@ static int handleListPlaylistInfo(int fd, int *permission,
 
 static int handleLsInfo(int fd, int *permission, int argc, char *argv[])
 {
-	if (argc == 1) {
-		if (printDirectoryInfo(fd, NULL) < 0)
-			return -1;
-		else
-			return lsPlaylists(fd, "");
-	} else {
-		if (printDirectoryInfo(fd, argv[1]) < 0)
-			return -1;
-		else
-			return lsPlaylists(fd, argv[1]);
-	}
+	char *path = "";
+
+	if (argc == 2)
+		path = argv[1];
+
+	if (printDirectoryInfo(fd, path) < 0)
+		return -1;
+
+	if (isRootDirectory(path))
+		return lsPlaylists(fd, path);
+
+	return 0;
 }
 
 static int handleRm(int fd, int *permission, int argc, char *argv[])
@@ -406,6 +422,11 @@ static int handleRm(int fd, int *permission, int argc, char *argv[])
 	return deletePlaylist(fd, argv[1]);
 }
 
+static int handleRename(int fd, int *permission, int argc, char *argv[])
+{
+	return renameStoredPlaylist(fd, argv[1], argv[2]);
+}
+
 static int handlePlaylistChanges(int fd, int *permission,
 				 int argc, char *argv[])
 {
@@ -508,13 +529,109 @@ static int handleSearch(int fd, int *permission, int argc, char *argv[])
 	return ret;
 }
 
+static int handleCount(int fd, int *permission, int argc, char *argv[])
+{
+	int ret;
+
+	LocateTagItem *items;
+	int numItems = newLocateTagItemArrayFromArgArray(argv + 1,
+							 argc - 1,
+							 &items);
+
+	if (numItems <= 0) {
+		commandError(fd, ACK_ERROR_ARG, "incorrect arguments");
+		return -1;
+	}
+
+	ret = searchStatsForSongsIn(fd, NULL, numItems, items);
+
+	freeLocateTagItemArray(numItems, items);
+
+	return ret;
+}
+
+static int handlePlaylistFind(int fd, int *permission, int argc, char *argv[])
+{
+	LocateTagItem *items;
+	int numItems = newLocateTagItemArrayFromArgArray(argv + 1,
+							 argc - 1,
+							 &items);
+
+	if (numItems <= 0) {
+		commandError(fd, ACK_ERROR_ARG, "incorrect arguments");
+		return -1;
+	}
+
+	findSongsInPlaylist(fd, numItems, items);
+
+	freeLocateTagItemArray(numItems, items);
+
+	return 0;
+}
+
+static int handlePlaylistSearch(int fd, int *permission, int argc, char *argv[])
+{
+	LocateTagItem *items;
+	int numItems = newLocateTagItemArrayFromArgArray(argv + 1,
+							 argc - 1,
+							 &items);
+
+	if (numItems <= 0) {
+		commandError(fd, ACK_ERROR_ARG, "incorrect arguments");
+		return -1;
+	}
+
+	searchForSongsInPlaylist(fd, numItems, items);
+
+	freeLocateTagItemArray(numItems, items);
+
+	return 0;
+}
+
+static int handlePlaylistDelete(int fd, int *permission, int argc, char *argv[]) {
+	char *playlist = argv[1];
+	int from;
+	char *test;
+
+	from = strtol(argv[2], &test, 10);
+	if (*test != '\0') {
+		commandError(fd, ACK_ERROR_ARG,
+			     "\"%s\" is not a integer", argv[2]);
+		return -1;
+	}
+
+	return removeOneSongFromStoredPlaylistByPath(fd, playlist, from);
+}
+
+static int handlePlaylistMove(int fd, int *permission, int argc, char *argv[])
+{
+	char *playlist = argv[1];
+	int from, to;
+	char *test;
+
+	from = strtol(argv[2], &test, 10);
+	if (*test != '\0') {
+		commandError(fd, ACK_ERROR_ARG,
+			     "\"%s\" is not a integer", argv[2]);
+		return -1;
+	}
+	to = strtol(argv[3], &test, 10);
+	if (*test != '\0') {
+		commandError(fd, ACK_ERROR_ARG,
+			     "\"%s\" is not a integer", argv[3]);
+		return -1;
+	}
+
+	return moveSongInStoredPlaylistByPath(fd, playlist, from, to);
+}
+
 static int listHandleUpdate(int fd,
 			    int *permission,
 			    int argc,
 			    char *argv[],
 			    struct strnode *cmdnode, CommandEntry * cmd)
 {
-	static List *pathList = NULL;
+	static List *pathList;
 	CommandEntry *nextCmd = NULL;
 	struct strnode *next = cmdnode->next;
 
@@ -646,6 +763,12 @@ static int handleList(int fd, int *permission, int argc, char *argv[])
 		return -1;
 	}
 
+	if (tagType == LOCATE_TAG_ANY_TYPE) {
+		commandError(fd, ACK_ERROR_ARG,
+		             "\"any\" is not a valid return tag type");
+		return -1;
+	}
+
 	/* for compatibility with < 0.12.0 */
 	if (argc == 3) {
 		if (tagType != TAG_ITEM_ALBUM) {
@@ -917,6 +1040,22 @@ static int handleNotcommands(int fd, int *permission, int argc, char *argv[])
 	return 0;
 }
 
+static int handlePlaylistClear(int fd, int *permission, int argc, char *argv[])
+{
+	return clearStoredPlaylist(fd, argv[1]);
+}
+
+static int handlePlaylistAdd(int fd, int *permission, int argc, char *argv[])
+{
+	char *playlist = argv[1];
+	char *path = argv[2];
+
+	if (isRemoteUrl(path))
+		return addToStoredPlaylist(fd, path, playlist);
+
+	return addAllInToStoredPlaylist(fd, path, playlist);
+}
+
 void initCommands(void)
 {
 	commandList = makeList(free, 1);
@@ -976,6 +1115,15 @@ void initCommands(void)
 	addCommand(COMMAND_DEVICES,          PERMISSION_READ,    0,   0,   handleDevices,              NULL);
 	addCommand(COMMAND_COMMANDS,         PERMISSION_NONE,    0,   0,   handleCommands,             NULL);
 	addCommand(COMMAND_NOTCOMMANDS,      PERMISSION_NONE,    0,   0,   handleNotcommands,          NULL);
+	addCommand(COMMAND_PLAYLISTCLEAR,    PERMISSION_CONTROL, 1,   1,   handlePlaylistClear,        NULL);
+	addCommand(COMMAND_PLAYLISTADD,      PERMISSION_CONTROL, 2,   2,   handlePlaylistAdd,          NULL);
+	addCommand(COMMAND_PLAYLISTFIND,     PERMISSION_READ,    2,   -1,  handlePlaylistFind,         NULL);
+	addCommand(COMMAND_PLAYLISTSEARCH,   PERMISSION_READ,    2,   -1,  handlePlaylistSearch,       NULL);
+	addCommand(COMMAND_PLAYLISTMOVE,     PERMISSION_CONTROL, 3,   3,   handlePlaylistMove,         NULL);
+	addCommand(COMMAND_PLAYLISTDELETE,   PERMISSION_CONTROL, 2,   2,   handlePlaylistDelete,       NULL);
+	addCommand(COMMAND_TAGTYPES,         PERMISSION_READ,    0,   0,   handleTagTypes,             NULL);
+	addCommand(COMMAND_COUNT,            PERMISSION_READ,    2,   -1,  handleCount,                NULL);
+	addCommand(COMMAND_RENAME,           PERMISSION_CONTROL, 2,   2,   handleRename,               NULL);
 
 	sortList(commandList);
 }
@@ -1134,15 +1282,18 @@ mpd_fprintf_ void commandError(int fd, int error, const char *fmt, ...)
 	va_list args;
 	va_start(args, fmt);
 
-	if (current_command) {
+	if (current_command && fd != STDERR_FILENO) {
 		fdprintf(fd, "ACK [%i@%i] {%s} ",
-			 (int)error, command_listNum, current_command);
+		         (int)error, command_listNum, current_command);
+		vfdprintf(fd, fmt, args);
+		fdprintf(fd, "\n");
 		current_command = NULL;
-	} else
+	} else {
 		fdprintf(STDERR_FILENO, "ACK [%i@%i] ",
-			 (int)error, command_listNum);
+		         (int)error, command_listNum);
+		vfdprintf(STDERR_FILENO, fmt, args);
+		fdprintf(STDERR_FILENO, "\n");
+	}
 
-	vfdprintf(fd, fmt, args);
 	va_end(args);
-	fdprintf(fd, "\n");
 }
