@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-static List *inputPlugin_list = NULL;
+static List *inputPlugin_list;
 
 void loadInputPlugin(InputPlugin * inputPlugin)
 {
@@ -59,7 +59,7 @@ static int stringFoundInStringArray(char **array, char *suffix)
 
 InputPlugin *getInputPluginFromSuffix(char *suffix, unsigned int next)
 {
-	static ListNode *pos = NULL;
+	static ListNode *pos;
 	ListNode *node;
 	InputPlugin *plugin;
 
@@ -88,7 +88,7 @@ InputPlugin *getInputPluginFromSuffix(char *suffix, unsigned int next)
 
 InputPlugin *getInputPluginFromMimeType(char *mimeType, unsigned int next)
 {
-	static ListNode *pos = NULL;
+	static ListNode *pos;
 	ListNode *node;
 	InputPlugin *plugin;
 
@@ -137,16 +137,6 @@ void printAllInputPluginSuffixes(FILE * fp)
 	fflush(fp);
 }
 
-extern InputPlugin mp3Plugin;
-extern InputPlugin oggvorbisPlugin;
-extern InputPlugin flacPlugin;
-extern InputPlugin oggflacPlugin;
-extern InputPlugin audiofilePlugin;
-extern InputPlugin mp4Plugin;
-extern InputPlugin mpcPlugin;
-extern InputPlugin aacPlugin;
-extern InputPlugin modPlugin;
-
 void initInputPlugins(void)
 {
 	inputPlugin_list = makeList(NULL, 1);
