@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -237,7 +237,7 @@ static float getAacFloatTotalTime(char *file)
 	size_t fileread, tagsize;
 	faacDecHandle decoder;
 	faacDecConfigurationPtr config;
-	unsigned int sampleRate;
+	unsigned long sampleRate;
 	unsigned char channels;
 	InputStream inStream;
 	long bread;
@@ -293,7 +293,7 @@ static int aac_decode(OutputBuffer * cb, DecoderControl * dc, char *path)
 	faacDecFrameInfo frameInfo;
 	faacDecConfigurationPtr config;
 	long bread;
-	unsigned int sampleRate;
+	unsigned long sampleRate;
 	unsigned char channels;
 	int eof = 0;
 	unsigned int sampleCount;
@@ -470,17 +470,6 @@ InputPlugin aacPlugin = {
 
 #else
 
-InputPlugin aacPlugin = {
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	0,
-	NULL,
-	NULL,
-};
+InputPlugin aacPlugin;
 
 #endif /* HAVE_FAAD */
