@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -100,7 +100,7 @@ static int mp4_decode(OutputBuffer * cb, DecoderControl * dc, char *path)
 	faacDecConfigurationPtr config;
 	unsigned char *mp4Buffer;
 	unsigned int mp4BufferSize;
-	uint32_t sampleRate;
+	unsigned long sampleRate;
 	unsigned char channels;
 	long sampleId;
 	long numSamples;
@@ -450,17 +450,6 @@ InputPlugin mp4Plugin = {
 
 #else
 
-InputPlugin mp4Plugin = {
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	0,
-	NULL,
-	NULL
-};
+InputPlugin mp4Plugin;
 
 #endif /* HAVE_FAAD */
