@@ -408,7 +408,7 @@ static unsigned int oggvorbis_try_decode(InputStream * inStream)
 	return (ogg_stream_type_detect(inStream) == VORBIS) ? 1 : 0;
 }
 
-static char *oggvorbis_Suffixes[] = { "ogg", NULL };
+static char *oggvorbis_Suffixes[] = { "ogg", "oga", NULL };
 static char *oggvorbis_MimeTypes[] = { "application/ogg",
                                        "audio/x-vorbis+ogg",
                                        "application/x-ogg",
