@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -47,10 +47,10 @@
 
 #define HTTP_REDIRECT_MAX    10
 
-static char *proxyHost = NULL;
-static char *proxyPort = NULL;
-static char *proxyUser = NULL;
-static char *proxyPassword = NULL;
+static char *proxyHost;
+static char *proxyPort;
+static char *proxyUser;
+static char *proxyPassword;
 static int bufferSize = HTTP_BUFFER_SIZE_DEFAULT;
 static int prebufferSize = HTTP_PREBUFFER_SIZE_DEFAULT;
 
@@ -81,9 +81,8 @@ void inputStream_initHttp(void)
 		param = getConfigParam(CONF_HTTP_PROXY_PORT);
 
 		if (!param) {
-			ERROR("%s specified but not %s", CONF_HTTP_PROXY_HOST,
+			FATAL("%s specified but not %s", CONF_HTTP_PROXY_HOST,
 			      CONF_HTTP_PROXY_PORT);
-			exit(EXIT_FAILURE);
 		}
 		proxyPort = param->value;
 
@@ -95,35 +94,30 @@ void inputStream_initHttp(void)
 			param = getConfigParam(CONF_HTTP_PROXY_PASSWORD);
 
 			if (!param) {
-				ERROR("%s specified but not %s\n",
+				FATAL("%s specified but not %s\n",
 				      CONF_HTTP_PROXY_USER,
 				      CONF_HTTP_PROXY_PASSWORD);
-				exit(EXIT_FAILURE);
 			}
 
 			proxyPassword = param->value;
-		}
+		} else {
+			param = getConfigParam(CONF_HTTP_PROXY_PASSWORD);
 
-		param = getConfigParam(CONF_HTTP_PROXY_PASSWORD);
-
-		if (param) {
-			ERROR("%s specified but not %s\n",
-			      CONF_HTTP_PROXY_PASSWORD, CONF_HTTP_PROXY_USER);
-			exit(EXIT_FAILURE);
+			if (param) {
+				FATAL("%s specified but not %s\n",
+				      CONF_HTTP_PROXY_PASSWORD, CONF_HTTP_PROXY_USER);
+			}
 		}
 	} else if ((param = getConfigParam(CONF_HTTP_PROXY_PORT))) {
-		ERROR("%s specified but not %s, line %i\n",
+		FATAL("%s specified but not %s, line %i\n",
 		      CONF_HTTP_PROXY_PORT, CONF_HTTP_PROXY_HOST, param->line);
-		exit(EXIT_FAILURE);
 	} else if ((param = getConfigParam(CONF_HTTP_PROXY_USER))) {
-		ERROR("%s specified but not %s, line %i\n",
+		FATAL("%s specified but not %s, line %i\n",
 		      CONF_HTTP_PROXY_USER, CONF_HTTP_PROXY_HOST, param->line);
-		exit(EXIT_FAILURE);
 	} else if ((param = getConfigParam(CONF_HTTP_PROXY_PASSWORD))) {
-		ERROR("%s specified but not %s, line %i\n",
+		FATAL("%s specified but not %s, line %i\n",
 		      CONF_HTTP_PROXY_PASSWORD, CONF_HTTP_PROXY_HOST,
 		      param->line);
-		exit(EXIT_FAILURE);
 	}
 
 	param = getConfigParam(CONF_HTTP_BUFFER_SIZE);
@@ -132,10 +126,9 @@ void inputStream_initHttp(void)
 		bufferSize = strtol(param->value, &test, 10);
 
 		if (bufferSize <= 0 || *test != '\0') {
-			ERROR("\"%s\" specified for %s at line %i is not a "
+			FATAL("\"%s\" specified for %s at line %i is not a "
 			      "positive integer\n",
 			      param->value, CONF_HTTP_BUFFER_SIZE, param->line);
-			exit(EXIT_FAILURE);
 		}
 
 		bufferSize *= 1024;
@@ -150,11 +143,10 @@ void inputStream_initHttp(void)
 		prebufferSize = strtol(param->value, &test, 10);
 
 		if (prebufferSize <= 0 || *test != '\0') {
-			ERROR("\"%s\" specified for %s at line %i is not a "
+			FATAL("\"%s\" specified for %s at line %i is not a "
 			      "positive integer\n",
 			      param->value, CONF_HTTP_PREBUFFER_SIZE,
 			      param->line);
-			exit(EXIT_FAILURE);
 		}
 
 		prebufferSize *= 1024;
@@ -446,7 +438,8 @@ static int finishHTTPInit(InputStream * inStream)
 	int error;
 	socklen_t error_len = sizeof(int);
 	int ret;
-	char request[2049];
+	int length;
+	char request[2048];
 
 	tv.tv_sec = 0;
 	tv.tv_usec = 0;
@@ -463,42 +456,41 @@ static int finishHTTPInit(InputStream * inStream)
 
 	if (ret < 0) {
 		DEBUG(__FILE__ ": problem select'ing: %s\n", strerror(errno));
-		close(data->sock);
-		data->connState = HTTP_CONN_STATE_CLOSED;
-		return -1;
+		goto close_err;
 	}
 
 	getsockopt(data->sock, SOL_SOCKET, SO_ERROR, &error, &error_len);
-	if (error) {
-		close(data->sock);
-		data->connState = HTTP_CONN_STATE_CLOSED;
-		return -1;
-	}
+	if (error)
+		goto close_err;
 
-	memset(request, 0, 2049);
 	/* deal with ICY metadata later, for now its fucking up stuff! */
-	snprintf(request, 2048, "GET %s HTTP/1.0\r\n" "Host: %s\r\n"
-		 /*"Connection: close\r\n" */
-		 "User-Agent: %s/%s\r\n"
-		 /*"Range: bytes=%ld-\r\n" */
-		 "%s"	/* authorization */
-		 "Icy-Metadata:1\r\n"
-		 "\r\n", data->path, data->host, PACKAGE_NAME, PACKAGE_VERSION,
-		 /*inStream->offset, */
-		 data->proxyAuth ? data->proxyAuth :
-		 (data->httpAuth ? data->httpAuth : "")
-	    );
+	length = snprintf(request, sizeof(request),
+	                 "GET %s HTTP/1.1\r\n" "Host: %s\r\n"
+			 /*"Connection: close\r\n" */
+			 "User-Agent: %s/%s\r\n"
+			 "Range: bytes=%ld-\r\n"
+			 "%s"	/* authorization */
+			 "Icy-Metadata:1\r\n"
+			 "\r\n",
+			 data->path, data->host,
+			 PACKAGE_NAME, PACKAGE_VERSION,
+			 inStream->offset,
+			 data->proxyAuth ? data->proxyAuth :
+			  (data->httpAuth ? data->httpAuth : ""));
 
-	ret = write(data->sock, request, strlen(request));
-	if (ret != strlen(request)) {
-		close(data->sock);
-		data->connState = HTTP_CONN_STATE_CLOSED;
-		return -1;
-	}
+	if (length >= sizeof(request))
+		goto close_err;
+	ret = write(data->sock, request, length);
+	if (ret != length)
+		goto close_err;
 
 	data->connState = HTTP_CONN_STATE_HELLO;
-
 	return 0;
+
+close_err:
+	close(data->sock);
+	data->connState = HTTP_CONN_STATE_CLOSED;
+	return -1;
 }
 
 static int getHTTPHello(InputStream * inStream)
@@ -681,9 +673,6 @@ static int getHTTPHello(InputStream * inStream)
 
 	data->prebuffer = 1;
 
-	/*mark as unseekable till we actually implement seeking */
-	inStream->seekable = 0;
-
 	return 0;
 }
 
@@ -714,21 +703,33 @@ int inputStream_httpOpen(InputStream * inStream, char *url)
 
 int inputStream_httpSeek(InputStream * inStream, long offset, int whence)
 {
-	/* hack to reopen an HTTP stream if we're trying to seek to
-	 * the beginning */
-	if ((whence == SEEK_SET) && (offset == 0)) {
-		InputStreamHTTPData *data;
+	InputStreamHTTPData *data;
 
-		data = (InputStreamHTTPData *) inStream->data;
-		close(data->sock);
-		data->connState = HTTP_CONN_STATE_REOPEN;
-		data->buflen = 0;
-		inStream->offset = 0;
-		return 0;
+	if (!inStream->seekable)
+		return -1;
+
+	switch (whence) {
+	case SEEK_SET:
+		inStream->offset = offset;
+		break;
+	case SEEK_CUR:
+		inStream->offset += offset;
+		break;
+	case SEEK_END:
+		inStream->offset = inStream->size + offset;
+		break;
+	default:
+		return -1;
 	}
 
-	/* otherwise, we don't know how to seek in HTTP yet */
-	return -1;
+	data = (InputStreamHTTPData *)inStream->data;
+	close(data->sock);
+	data->connState = HTTP_CONN_STATE_REOPEN;
+	data->buflen = 0;
+
+	inputStream_httpBuffer(inStream);
+
+	return 0;
 }
 
 static void parseIcyMetadata(InputStream * inStream, char *metadata, int size)
