@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -50,7 +50,7 @@ static void blockingWrite(const int fd, const char *string, size_t len)
 
 void vfdprintf(const int fd, const char *fmt, va_list args)
 {
-	static char buffer[BUFFER_LENGTH + 1];
+	static char buffer[BUFFER_LENGTH];
 	char *buf = buffer;
 	size_t len;
 
