@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,7 @@
 static mpd_sint16 currentChunk = -1;
 
 static mpd_sint8 currentMetaChunk = -1;
-static mpd_sint8 sendMetaChunk = 0;
+static mpd_sint8 sendMetaChunk;
 
 void clearAllMetaChunkSets(OutputBuffer * cb)
 {
@@ -58,11 +58,10 @@ void clearOutputBuffer(OutputBuffer * cb)
 void flushOutputBuffer(OutputBuffer * cb)
 {
 	if (currentChunk == cb->end) {
-		int next = cb->end + 1;
-		if (next >= buffered_chunks) {
-			next = 0;
+		if ((cb->end + 1) >= buffered_chunks) {
+			cb->end = 0;
 		}
-		cb->end = next;
+		else cb->end++;
 		currentChunk = -1;
 	}
 }
@@ -76,27 +75,23 @@ int sendDataToOutputBuffer(OutputBuffer * cb, InputStream * inStream,
 	mpd_uint16 chunkLeft;
 	char *data;
 	size_t datalen;
-	static char *convBuffer = NULL;
-	static long convBufferLen = 0;
+	static char *convBuffer;
+	static long convBufferLen;
 
 	if (cmpAudioFormat(&(cb->audioFormat), &(dc->audioFormat)) == 0) {
 		data = dataIn;
 		datalen = dataInLen;
 	} else {
-		datalen =
-		    pcm_sizeOfOutputBufferForAudioFormatConversion(&
-								   (dc->
-								    audioFormat),
-								   dataInLen,
-								   &(cb->
-								     audioFormat));
+		datalen = pcm_sizeOfConvBuffer(&(dc->audioFormat), dataInLen,
+		                               &(cb->audioFormat));
 		if (datalen > convBufferLen) {
 			convBuffer = xrealloc(convBuffer, datalen);
 			convBufferLen = datalen;
 		}
 		data = convBuffer;
-		pcm_convertAudioFormat(&(dc->audioFormat), dataIn, dataInLen,
-				       &(cb->audioFormat), data);
+		datalen = pcm_convertAudioFormat(&(dc->audioFormat), dataIn,
+		                                 dataInLen, &(cb->audioFormat),
+		                                 data, &(cb->convState));
 	}
 
 	if (replayGainInfo && (replayGainState != REPLAYGAIN_OFF))
@@ -158,7 +153,7 @@ int sendDataToOutputBuffer(OutputBuffer * cb, InputStream * inStream,
 int copyMpdTagToOutputBuffer(OutputBuffer * cb, MpdTag * tag)
 {
 	int nextChunk;
-	static MpdTag *last = NULL;
+	static MpdTag *last;
 
 	if (!cb->acceptMetadata || !tag) {
 		sendMetaChunk = 0;
