@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,6 +19,7 @@
 #ifndef OUTPUT_BUFFER_H
 #define OUTPUT_BUFFER_H
 
+#include "pcm_utils.h"
 #include "mpd_types.h"
 #include "decode.h"
 #include "audio.h"
@@ -39,6 +40,7 @@ typedef struct _OutputBuffer {
 	mpd_sint16 volatile begin;
 	mpd_sint16 volatile end;
 	AudioFormat audioFormat;
+	ConvState convState;
 	MetadataChunk metadataChunks[BUFFERED_METACHUNKS];
 	mpd_sint8 metaChunkSet[BUFFERED_METACHUNKS];
 	mpd_sint8 *volatile metaChunk;
