@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -40,7 +40,7 @@
 
 const char *musicDir;
 static const char *playlistDir;
-static char *fsCharset = NULL;
+static char *fsCharset;
 
 static char *pathConvCharset(char *to, char *from, char *str, char *ret)
 {
@@ -51,7 +51,7 @@ static char *pathConvCharset(char *to, char *from, char *str, char *ret)
 
 char *fsCharsetToUtf8(char *str)
 {
-	static char *ret = NULL;
+	static char *ret;
 
 	ret = pathConvCharset("UTF-8", fsCharset, str, ret);
 
@@ -65,7 +65,7 @@ char *fsCharsetToUtf8(char *str)
 
 char *utf8ToFsCharset(char *str)
 {
-	static char *ret = NULL;
+	static char *ret;
 
 	ret = pathConvCharset(fsCharset, "UTF-8", str, ret);
 
@@ -142,18 +142,16 @@ void initPaths(void)
 	playlistDir = appendSlash(&(playlistParam->value));
 
 	if ((dir = opendir(playlistDir)) == NULL) {
-		ERROR("cannot open %s \"%s\" (config line %i): %s\n",
+		FATAL("cannot open %s \"%s\" (config line %i): %s\n",
 		      CONF_PLAYLIST_DIR, playlistParam->value,
 		      playlistParam->line, strerror(errno));
-		exit(EXIT_FAILURE);
 	}
 	closedir(dir);
 
 	if ((dir = opendir(musicDir)) == NULL) {
-		ERROR("cannot open %s \"%s\" (config line %i): %s\n",
+		FATAL("cannot open %s \"%s\" (config line %i): %s\n",
 		      CONF_MUSIC_DIR, musicParam->value,
 		      musicParam->line, strerror(errno));
-		exit(EXIT_FAILURE);
 	}
 	closedir(dir);
 
@@ -281,12 +279,12 @@ char *sanitizePathDup(char *path)
 
 	len = 0;
 
-	/* illeminate more than one '/' in a row, like "///" */
+	/* eliminate more than one '/' in a row, like "///" */
 	while (*path) {
 		while (*path == '/')
 			path++;
 		if (*path == '.') {
-			/* we dont want to have hidden directoires, or '.' or
+			/* we don't want to have hidden directories, or '.' or
 			   ".." in our path */
 			free(ret);
 			return NULL;
