@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,9 @@
 #define SONG_TYPE_FILE 1
 #define SONG_TYPE_URL 2
 
+#define SONG_FILE	"file: "
+#define SONG_TIME	"Time: "
+
 typedef struct _Song {
 	char *url;
 	mpd_sint8 type;
