@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -41,7 +41,7 @@ static struct _sf_cb {
 	{ readPlaylistState, savePlaylistState },
 };
 
-static const char *sfpath = NULL;
+static const char *sfpath;
 
 static void get_state_file_path(void)
 {
@@ -87,16 +87,13 @@ void read_state_file(void)
 		DEBUG("failed to stat state file: %s\n", sfpath);
 		return;
 	}
-	if (!S_ISREG(st.st_mode)) {
-		ERROR("state file \"%s\" is not a regular file\n", sfpath);
-		exit(EXIT_FAILURE);
-	}
+	if (!S_ISREG(st.st_mode))
+		FATAL("state file \"%s\" is not a regular file\n", sfpath);
 
 	while (!(fp = fopen(sfpath, "r")) && errno == EINTR);
 	if (mpd_unlikely(!fp)) {
-		ERROR("problems opening state file \"%s\" for reading: %s\n",
+		FATAL("problems opening state file \"%s\" for reading: %s\n",
 		      sfpath, strerror(errno));
-		exit(EXIT_FAILURE);
 	}
 	for (i = 0; i < ARRAY_SIZE(sf_callbacks); i++) {
 		sf_callbacks[i].reader(fp);
@@ -108,7 +105,7 @@ void read_state_file(void)
 
 void mpd_noreturn state_file_fatal(void)
 {
-	ERROR("error parsing state file \"%s\"\n", sfpath);
+	FATAL("error parsing state file \"%s\"\n", sfpath);
 	exit(EXIT_FAILURE);
 }
 
